package com.cloudpath.iam.admin.service.lingyun.impl;

package ..deTemp.1729608947054;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cloudpath.commons.constant.IsDelEnum;
import com.cloudpath.commons.constant.LogTypeEnum;
import com.cloudpath.commons.exception.BusinessException;
import com.cloudpath.commons.service.MailService;
import com.cloudpath.commons.util.AESUtil;
import com.cloudpath.commons.util.LocalDateTimeUtil;
import com.cloudpath.commons.util.PasswordUtils;
import com.cloudpath.commons.util.SaltUtils;
import com.cloudpath.commons.util.UpdateLogUtil;
import com.cloudpath.commons.util.common.Tools;
import com.cloudpath.iam.admin.authentication.AuthenticationService;
import com.cloudpath.iam.admin.authentication.CommonPermission;
import com.cloudpath.iam.admin.config.RefreshScopeConfig;
import com.cloudpath.iam.admin.constant.AdminRoleEnum;
import com.cloudpath.iam.admin.constant.Constant;
import com.cloudpath.iam.admin.constant.OrgStatusEnum;
import com.cloudpath.iam.admin.constant.SourceFromEnum;
import com.cloudpath.iam.admin.constant.StaffStatusEnum;
import com.cloudpath.iam.admin.domain.AccessPolicy;
import com.cloudpath.iam.admin.domain.AdminRole;
import com.cloudpath.iam.admin.domain.AdminStaffOrg;
import com.cloudpath.iam.admin.domain.StaffPolicyAssociation;
import com.cloudpath.iam.admin.domain.UcOrganization;
import com.cloudpath.iam.admin.domain.UcStaff;
import com.cloudpath.iam.admin.domain.UcStaffExample;
import com.cloudpath.iam.admin.domain.UcStaffOrg;
import com.cloudpath.iam.admin.dto.AuthUcStaffDto;
import com.cloudpath.iam.admin.dto.EmailDto;
import com.cloudpath.iam.admin.dto.OrgTenantDto;
import com.cloudpath.iam.admin.dto.StaffDto;
import com.cloudpath.iam.admin.dto.StaffOrgPermissionDto;
import com.cloudpath.iam.admin.dto.StaffUpdateDto;
import com.cloudpath.iam.admin.dto.StaffsDto;
import com.cloudpath.iam.admin.feign.BssService;
import com.cloudpath.iam.admin.feign.IBasicInterfaceInfoService;
import com.cloudpath.iam.admin.feign.WfeService;
import com.cloudpath.iam.admin.mapper.AccessPolicyMapper;
import com.cloudpath.iam.admin.mapper.AdminStaffOrgMapper;
import com.cloudpath.iam.admin.mapper.StaffPolicyAssociationMapper;
import com.cloudpath.iam.admin.mapper.UcStaffMapper;
import com.cloudpath.iam.admin.service.AdminRoleService;
import com.cloudpath.iam.admin.service.SysOperateLogService;
import com.cloudpath.iam.admin.service.UcOrganizationService;
import com.cloudpath.iam.admin.service.UcStaffOrgService;
import com.cloudpath.iam.admin.service.lingyun.StaffService;
import com.cloudpath.iam.admin.service.sign.ISignAuthUtilService;
import com.cloudpath.iam.admin.util.CaptchaUtils;
import com.cloudpath.iam.admin.util.CommonUser;
import com.cloudpath.iam.admin.util.FailCode;
import com.cloudpath.iam.admin.util.RedisCacheKey;
import com.cloudpath.iam.admin.util.RedisUtil;
import com.cloudpath.iam.admin.util.cache.OrgTreeCache;
import com.cloudpath.iam.admin.util.template.Email;
import com.cloudpath.iam.admin.vo.bss.MasterTenantVo;
import com.cloudpath.iam.admin.vo.permission.StaffDataPermission;
import com.cloudpath.iam.admin.vo.staff.PermissionTenant;
import com.cloudpath.iam.admin.vo.staff.StaffOrganization;
import com.google.common.collect.Lists;
import java.io.ByteArrayOutputStream;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import jodd.util.StringUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

@Service
public class StaffServiceImpl implements StaffService {
  private static final Logger log = LogManager.getLogger(com.cloudpath.iam.admin.service.lingyun.impl.StaffServiceImpl.class);
  
  @Resource
  private UcStaffMapper ucStaffMapper;
  
  @Autowired
  private AuthenticationService authenticationService;
  
  @Autowired
  private AdminRoleService adminRoleService;
  
  @Autowired
  private RedisUtil redisUtil;
  
  @Autowired
  private AccessPolicyMapper accessPolicyMapper;
  
  @Autowired
  private StaffPolicyAssociationMapper associationMapper;
  
  @Autowired
  private UcStaffOrgService ucStaffOrgService;
  
  @Autowired
  private UcOrganizationService ucOrganizationService;
  
  @Autowired
  private SysOperateLogService sysOperateLogService;
  
  @Autowired
  RefreshScopeConfig refreshScopeConfig;
  
  @Autowired
  private MailService mailService;
  
  @Autowired
  private BssService bssService;
  
  @Autowired
  private WfeService wfeService;
  
  @Value("${mail.effective-time}")
  private Long effectiveTime;
  
  @Autowired
  private IBasicInterfaceInfoService cmdbService;
  
  @Autowired
  private OrgTreeCache orgTreeCache;
  
  @Autowired
  private AdminStaffOrgMapper adminStaffOrgMapper;
  
  @Autowired
  private CommonPermission permission;
  
  @Value("${auth.aesKey:U4ZTJiMzMtNmZjZC}")
  private String aesKey;
  
  @Value("${auth.aesIv:BiMzVmYjctOGM2Zi}")
  private String aesIv;
  
  private final String code = "code";
  
  private final String msg = "msg";
  
  private final int successCode = 200;
  
  @Value("${encryptType}")
  private String encryptType;
  
  @Autowired
  private ISignAuthUtilService signAuthUtilService;
  
  @Transactional(rollbackFor = {Exception.class})
  public Long addStaff(UcStaff ucStaff, List<UcStaffOrg> staffOrgs, StaffDto dto) {
    UcStaff staff = getStaffByLoginName(ucStaff.getLoginName());
    if (null != staff) {
      log.info("用户信息已经存在,loginName:{}", ucStaff.getLoginName());
      throw new BusinessException(FailCode.STAFF_ADD_NAME_REPEAT.getCode(), FailCode.STAFF_ADD_NAME_REPEAT
          .getMessage());
    } 
    if (!StringUtils.isEmpty(ucStaff.getStaffEmail()) && 
      null != this.ucStaffMapper.getStaffByEmail(ucStaff.getStaffEmail())) {
      log.info("邮箱已存在,:{}", ucStaff.getStaffEmail());
      throw new BusinessException(FailCode.STAFF_ADD_NAME_REPEAT.getCode(), "邮箱已存在");
    } 
    checkOrg(staffOrgs);
    int i = insertStaff(ucStaff, dto);
    if (i > 0) {
      this.ucStaffMapper.updateByPrimaryKeySelective(ucStaff);
      Long staffId = ucStaff.getStaffId();
      if (!CollectionUtils.isEmpty(staffOrgs)) {
        log.info("保存用户岗位信息,staffOrgs:{}", JSON.toJSONString(staffOrgs));
        for (UcStaffOrg staffOrg : staffOrgs) {
          UcOrganization orgByOrgCode = this.ucOrganizationService.getOrgByOrgCode(staffOrg.getOrgCode());
          if (orgByOrgCode == null) {
            log.info("新增用户关联组织代码不存在,staffOrgs:{}", JSON.toJSONString(staffOrgs));
            throw new BusinessException(FailCode.ORG_NOT_EXISTS.getCode(), FailCode.ORG_NOT_EXISTS
                .getMessage());
          } 
          addStaffOrg(ucStaff, staffOrg);
        } 
      } 
      boolean feign = dto.isFeign();
      if (!feign && !StringUtils.isEmpty(dto.getMasterTenantId())) {
        JSONObject result = this.bssService.masterTenantStaffRela(Long.valueOf(dto.getMasterTenantId()), staffId);
        if (!result.containsKey("code") || 200 != result.getInteger("code").intValue()) {
          log.warn("调用远程接口关联用户与主租户失败，staffId:{},结果：{}", staffId, result);
          throw new BusinessException(FailCode.API_EXCEPTION.getCode(), result.getString("msg"));
        } 
      } 
      this.orgTreeCache.delOrgTreeCache();
      if (ucStaff.isDefaultPolicy())
        bindDefaultPolicy(ucStaff); 
      log.info("新增用户数据成功，ucStaff:{}, staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
      return staffId;
    } 
    log.info("新增用户数据失败，ucStaff:{}, staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
    return null;
  }
  
  private void unbindDefaultPolicy(UcStaff ucstaff) {
    List<AccessPolicy> defaultPolicy = this.accessPolicyMapper.selectDefaultPolicy();
    defaultPolicy.forEach(policy -> {
          StaffPolicyAssociation association = new StaffPolicyAssociation();
          association.setUpdateTime(new Date());
          association.setStaffId(ucstaff.getStaffId().toString());
          association.setIsDel(1);
          association.setPolicyId(policy.getPolicyId());
          log.info("【解绑默认策略:{}】", JSON.toJSONString(association));
          List<StaffPolicyAssociation> association1 = this.associationMapper.selectByAssociation(association);
          if (association1.size() > 0)
            this.associationMapper.updateByPrimaryKeySelective(association); 
        });
  }
  
  private void bindDefaultPolicy(UcStaff ucstaff) {
    List<StaffPolicyAssociation> associations = this.associationMapper.selectByStaffId(ucstaff.getStaffId().toString());
    associations.forEach(association -> {
          association.setIsDel(1);
          this.associationMapper.updateByPrimaryKeySelective(association);
        });
    List<AccessPolicy> defaultPolicy = this.accessPolicyMapper.selectDefaultPolicy();
    defaultPolicy.forEach(policy -> {
          StaffPolicyAssociation association = new StaffPolicyAssociation();
          association.setCreateTime(new Date());
          association.setUpdateTime(new Date());
          association.setDescription("新增人员绑定");
          association.setStaffId(ucstaff.getStaffId().toString());
          association.setIsDel(0);
          association.setPolicyId(policy.getPolicyId());
          log.info("【绑定默认策略:{}】", JSON.toJSONString(association));
          List<StaffPolicyAssociation> association1 = this.associationMapper.selectByAssociation(association);
          if (association1.isEmpty())
            this.associationMapper.insert(association); 
        });
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public Long addStaffNoCheckOrg(UcStaff ucStaff, List<UcStaffOrg> staffOrgs, StaffDto dto) {
    UcStaff staff = getStaffByLoginName(ucStaff.getLoginName());
    if (null != staff) {
      log.info("用户信息已经存在,loginName:{}", ucStaff.getLoginName());
      throw new BusinessException(FailCode.STAFF_ADD_NAME_REPEAT.getCode(), FailCode.STAFF_ADD_NAME_REPEAT
          .getMessage());
    } 
    int i = insertStaff(ucStaff, dto);
    if (i > 0) {
      this.ucStaffMapper.updateByPrimaryKeySelective(ucStaff);
      Long staffId = ucStaff.getStaffId();
      if (!CollectionUtils.isEmpty(staffOrgs)) {
        if (StringUtils.isEmpty(((UcStaffOrg)staffOrgs.get(0)).getOrgCode()))
          ((UcStaffOrg)staffOrgs.get(0)).setOrgCode("同步用户orgCode为null"); 
        log.info("保存用户岗位信息,staffOrgs:{}", JSON.toJSONString(staffOrgs));
        for (UcStaffOrg staffOrg : staffOrgs)
          addStaffOrg(ucStaff, staffOrg); 
      } 
      this.orgTreeCache.delOrgTreeCache();
      log.info("新增用户数据成功，ucStaff:{}, staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
      return staffId;
    } 
    log.info("新增用户数据失败，ucStaff:{}, staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
    return null;
  }
  
  private int insertStaff(UcStaff ucStaff, StaffDto dto) {
    ucStaff.setUserId(UUID.randomUUID().toString());
    ucStaff.setStaffType(Short.valueOf((ucStaff.getStaffType() == null) ? 1 : ucStaff.getStaffType().shortValue()));
    String createBy = CommonUser.currentUser().getLoginName();
    ucStaff.setCreateBy(((createBy == null) ? dto.getCreateBy() : createBy) + "创建新增");
    ucStaff.setCreateTime(LocalDateTimeUtil.UDateToLocalDateTime(new Date()));
    return this.ucStaffMapper.insert(ucStaff);
  }
  
  private void checkOrg(List<UcStaffOrg> staffOrgs) {
    if (!CollectionUtils.isEmpty(staffOrgs) && !StringUtils.isEmpty(((UcStaffOrg)staffOrgs.get(0)).getOrgCode())) {
      UcOrganization parentOrg = this.ucOrganizationService.getOrgByOrgCode(((UcStaffOrg)staffOrgs.get(0)).getOrgCode());
      if (parentOrg == null) {
        log.info("添加组织机构数据失败,获取上级机构失败，org:{}", JSON.toJSONString(staffOrgs));
        throw new BusinessException(FailCode.ORG_PARENT_NOT_EXIT.getCode(), FailCode.ORG_PARENT_NOT_EXIT.getMessage());
      } 
      if (parentOrg.getOrgStatus() != OrgStatusEnum.VALID) {
        log.info("组织父机构不是启用状态，org:{}", JSON.toJSONString(staffOrgs));
        throw new BusinessException(FailCode.STAFF_ORG_PARENTS_NOT_ACTIVE.getCode(), FailCode.STAFF_ORG_PARENTS_NOT_ACTIVE.getMessage());
      } 
    } 
  }
  
  private void addStaffOrg(UcStaff ucStaff, UcStaffOrg staffOrg) {
    staffOrg.setStaffId(ucStaff.getStaffId());
    if (staffOrg.getCreateBy() == null || staffOrg.getCreateBy().isEmpty())
      staffOrg.setCreateBy(ucStaff.getCreateBy()); 
    boolean soResult = this.ucStaffOrgService.addStaffOrg(staffOrg);
    if (!soResult) {
      log.info("新增用户与组织关联失败,staffOrgs:{}", JSON.toJSONString(staffOrg));
      throw new BusinessException(FailCode.STAFF_ORG_ADD_BIND_FAIL.getCode(), FailCode.STAFF_ORG_ADD_BIND_FAIL
          .getMessage());
    } 
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public UcStaff updateStaffByStaffId(UcStaff ucStaff, List<UcStaffOrg> staffOrgs, StaffDto dto) {
    if (!StringUtils.isEmpty(ucStaff.getPassword()) && this.refreshScopeConfig.getUseEmail().booleanValue())
      verifyCode(ucStaff); 
    checkOrg(staffOrgs);
    UcStaff loginStaff = getLoginStaff();
    UcStaff staff = validParams(loginStaff, ucStaff.getStaffId(), null, false, null, false, true);
    ucStaff.setLastModifyBy(CommonUser.currentUser().getLoginName());
    ucStaff.setLastModifyTime(LocalDateTimeUtil.UDateToLocalDateTime(new Date()));
    ucStaff.setLoginName(null);
    ucStaff.setUserId(null);
    int result = this.ucStaffMapper.updateByPrimaryKeySelective(ucStaff);
    if (ucStaff.isDefaultPolicy()) {
      bindDefaultPolicy(ucStaff);
    } else {
      unbindDefaultPolicy(ucStaff);
    } 
    if (result > 0) {
      log.info("保存用户岗位信息,staffOrgs:{}", JSON.toJSONString(staffOrgs));
      if (ucStaff.getStaffStatus() != null)
        updateStaffStatus(ucStaff.getStaffId(), ucStaff.getStaffStatus().value()); 
      changeStaffMasterTenantRela(ucStaff, dto);
      UcStaff newStaff = (UcStaff)this.ucStaffMapper.selectByPrimaryKey(ucStaff.getStaffId());
      this.authenticationService.updateStaffPhoneAndEmail(staff.getStaffId(), newStaff.getStaffPhone(), newStaff.getStaffEmail());
      log.info("修改用户数据成功，staff:{},staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
      return staff;
    } 
    log.info("修改用户数据失败，staff:{},staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
    throw new BusinessException(FailCode.ORG_PARENT_NOT_EXIT.getCode(), FailCode.ORG_PARENT_NOT_EXIT.getMessage());
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public void updateStaffs(StaffsDto staffsDto) {
    int key = staffsDto.getAccessType();
    log.info("【门户设置全局认证】:{}", Integer.valueOf(key));
    this.redisUtil.set("door.access.switch", Integer.valueOf(key));
    List<StaffUpdateDto> staffUpdateDtos = staffsDto.getStaffs();
    staffUpdateDtos.forEach(staff -> {
          UcStaff ucStaff = new UcStaff();
          ucStaff.setStaffId(staff.getStaffId());
          ucStaff.setIdcardNo(staff.getIdCardNo());
          this.ucStaffMapper.updateByStaffId(ucStaff);
        });
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public boolean updateStaffSync(UcStaff ucStaff, List<UcStaffOrg> staffOrgs, StaffDto dto) {
    ucStaff.setLastModifyBy(CommonUser.currentUser().getLoginName());
    ucStaff.setLastModifyTime(LocalDateTimeUtil.UDateToLocalDateTime(new Date()));
    ucStaff.setLoginName(null);
    ucStaff.setUserId(null);
    int result = this.ucStaffMapper.updateByPrimaryKeySelective(ucStaff);
    if (result > 0) {
      this.ucStaffOrgService.delAllStaffOrgByStaffId(ucStaff.getStaffId());
      if (!CollectionUtils.isEmpty(staffOrgs) && !StringUtils.isEmpty(((UcStaffOrg)staffOrgs.get(0)).getOrgCode())) {
        log.info("保存用户岗位信息,staffOrgs:{}", JSON.toJSONString(staffOrgs));
        for (UcStaffOrg staffOrg : staffOrgs)
          addStaffOrg(ucStaff, staffOrg); 
      } 
      changeStaffMasterTenantRela(ucStaff, dto);
      log.info("修改用户数据成功，staff:{},staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
      if (ucStaff.isDefaultPolicy())
        bindDefaultPolicy(ucStaff); 
      return true;
    } 
    log.info("修改用户数据失败，staff:{},staffOrgs:{}", JSON.toJSONString(ucStaff), JSON.toJSONString(staffOrgs));
    return false;
  }
  
  private void changeStaffMasterTenantRela(UcStaff ucStaff, StaffDto dto) {
    if (dto.getMasterTenantId() != null) {
      JSONObject param = new JSONObject();
      param.put("staffId", ucStaff.getStaffId());
      param.put("masterTenantId", dto.getMasterTenantId());
      JSONObject udpMasterTenantStaffRelaResult = this.bssService.udpMasterTenantStaffRelaByStaffId(param);
      if (!udpMasterTenantStaffRelaResult.containsKey("code") || 200 != udpMasterTenantStaffRelaResult.getInteger("code").intValue()) {
        log.error("调用远程接口修改用户与主租户失败，staffId:{},结果：{}", ucStaff.getStaffId(), udpMasterTenantStaffRelaResult);
        throw new BusinessException(FailCode.API_EXCEPTION.getCode(), udpMasterTenantStaffRelaResult.getString("msg"));
      } 
    } 
  }
  
  private void delAndInsertStaffOrg(UcStaff ucStaff, List<UcStaffOrg> staffOrgs) {
    this.ucStaffOrgService.delAllStaffOrgByStaffId(ucStaff.getStaffId());
    for (UcStaffOrg staffOrg : staffOrgs) {
      UcOrganization orgByOrgCode = this.ucOrganizationService.getOrgByOrgCode(staffOrg.getOrgCode());
      if (orgByOrgCode == null) {
        log.info("新增用户关联组织代码不存在,staffOrgs:{}", JSON.toJSONString(staffOrgs));
        throw new BusinessException(FailCode.ORG_NOT_EXISTS.getCode(), FailCode.ORG_NOT_EXISTS
            .getMessage());
      } 
      addStaffOrg(ucStaff, staffOrg);
    } 
  }
  
  private void verifyCode(UcStaff ucStaff) {
    String verifyCode = ucStaff.getVerifyCode();
    String redisKey = RedisCacheKey.getVerifyCode(CommonUser.currentUser().getLoginName() + "," + 
        CommonUser.currentUser().getStaffEmail());
    if (!this.redisUtil.hasKey(redisKey))
      throw new BusinessException(FailCode.RESET_PASSWORD_VERIFY_OVERDUE.getCode(), FailCode.RESET_PASSWORD_VERIFY_OVERDUE
          .getMessage()); 
    String redisVerifyCode = String.valueOf(this.redisUtil.get(redisKey));
    if (!verifyCode.equals(redisVerifyCode))
      throw new BusinessException(FailCode.RESET_PASSWORD_VERIFY_FAIL.getCode(), FailCode.RESET_PASSWORD_VERIFY_FAIL
          .getMessage()); 
  }
  
  public UcStaff updateStaffStatus(Long staffId, Short staffStatus) {
    UcStaff loginStaff = getLoginStaff();
    UcStaff staff = validParams(loginStaff, staffId, staffStatus, true, null, false, false);
    if (staff.getStaffStatus() != null && staff.getStaffStatus().value() != null && 
      staff.getStaffStatus().value().equals(staffStatus)) {
      log.info("updateStaffStatus与数据库状态一致,loginStaffId:{},staffId:{},staffStatus:{}", loginStaff
          .getStaffId(), staffId, staffStatus);
      return staff;
    } 
    UcStaff upStaff = new UcStaff();
    if (StaffStatusEnum.FORBID.isSame(staffStatus))
      upStaff.setStaffStatus(StaffStatusEnum.FORBID); 
    if (StaffStatusEnum.USE.isSame(staffStatus))
      upStaff.setStaffStatus(StaffStatusEnum.USE); 
    upStaff.setStaffId(staffId);
    upStaff.setLastModifyBy(loginStaff.getLoginName());
    upStaff.setLastModifyTime(LocalDateTime.now());
    int num = this.ucStaffMapper.updateByPrimaryKeySelective(upStaff);
    log.info("updateStaffStatus更新条数:{}loginStaffId:{},staffId:{},staffStatus:{}", Integer.valueOf(num), loginStaff.getStaffId(), staffId, staffStatus);
    if (num > 0) {
      if (StaffStatusEnum.FORBID.isSame(staffStatus))
        try {
          removeTokenAndPermissionCache(Collections.singletonList(staffId));
        } catch (Exception e) {
          log.info("设置权限清空token缓存失败staffId:{},staffStatus:{},异常:{}", staffId, staffStatus, e);
        }  
      try {
        this.sysOperateLogService.asyncAddOperateLog(this.sysOperateLogService.generateSysOperateLogWithBLOBs(LogTypeEnum.UPDATE, "uc_staff", null, UpdateLogUtil.UpdateLog(staff, upStaff)));
      } catch (Exception e) {
        log.info("updateStaffStatus记录操作日志失败staffId:{},staffStatus:{},异常:{}", staffId, staffStatus, e);
      } 
      if (StaffStatusEnum.FORBID.isSame(staffStatus))
        try {
          removeTokenAndPermissionCache(Collections.singletonList(staffId));
        } catch (Exception e) {
          log.info("设置权限清空token缓存失败staffId:{},staffStatus:{},异常:{}", staffId, staffStatus, e);
        }  
    } else {
      log.info("updateStaffStatus更新条数:{},loginStaffId:{},staffId:{},staffStatus:{}", Integer.valueOf(num), loginStaff.getStaffId(), staffId, staffStatus);
      throw new BusinessException(FailCode.QUERY_STAFF_FAIL_OPT.getCode(), FailCode.QUERY_STAFF_FAIL_OPT.getMessage());
    } 
    return staff;
  }
  
  private void checkWorkflow(Long staffId) {
    JSONObject param = new JSONObject();
    param.put("applyUserId", staffId);
    param.put("processStatus", Integer.valueOf(1));
    JSONObject taskInfoResult = this.wfeService.getTaskInfo(param);
    if (!taskInfoResult.containsKey("status") || !"200".equals(taskInfoResult.get("status"))) {
      log.error("获取用户正在进行的工单失败，staffId:{},结果：{}", staffId, taskInfoResult);
      throw new BusinessException(FailCode.API_EXCEPTION.getCode(), taskInfoResult.getString("msg"));
    } 
    if (!"0".equals(taskInfoResult.getJSONObject("data").getString("total")))
      throw new BusinessException(FailCode.API_EXCEPTION.getCode(), "该用户有正在进行的工单/订单，无法停用！"); 
    log.info("获取workflowservice用户正在进行的工单 ,结果：{}", taskInfoResult);
    Map<String, Object> queryMap = new HashMap<>();
    queryMap.put("staffId", staffId);
    queryMap.put("status", "0,1");
    queryMap.put("pageNumber", Integer.valueOf(1));
    queryMap.put("pageSize", Integer.valueOf(1));
    JSONObject allWorkOrders = this.cmdbService.getAllWorkOrders(queryMap);
    if (!allWorkOrders.containsKey("code") || !"200".equals(allWorkOrders.getInteger("code").toString())) {
      log.error(allWorkOrders.toJSONString());
      throw new BusinessException(FailCode.API_EXCEPTION.getCode(), "获取工单失败");
    } 
    if (allWorkOrders.containsKey("total") && allWorkOrders.getInteger("total").intValue() > 0)
      throw new BusinessException(FailCode.API_EXCEPTION.getCode(), "该人员有正在进行的工单，无法停用!"); 
  }
  
  public boolean sendLoginUserVerifyCode() {
    String email = CommonUser.currentUser().getStaffEmail();
    String loginName = CommonUser.currentUser().getLoginName();
    log.info("修改人员发送邮件验证码开始，loginName:{},mail:{}", loginName, email);
    String redisKey = RedisCacheKey.getVerifyCode(loginName + "," + email);
    boolean hasKey = this.redisUtil.hasKey(redisKey);
    if (hasKey)
      throw new BusinessException(FailCode.RESET_PASSWORD_SEND_EMAIL_HARD.getCode(), FailCode.RESET_PASSWORD_SEND_EMAIL_HARD
          .getMessage()); 
    String captcha = CaptchaUtils.captcha(8);
    try {
      String subject = "修改人员信息";
      String emailHtml = Email.getEmailHtml(subject, captcha);
      this.mailService.sendMail(email, subject, emailHtml);
    } catch (Exception e) {
      log.error("发送邮件异常", e);
    } 
    this.redisUtil.set(redisKey, captcha, this.effectiveTime.longValue());
    log.info("修改人员发送邮件验证码结束，loginName:{},mail:{},redisKey:{},verifyCode:{}", loginName, email, redisKey, captcha);
    return true;
  }
  
  private UcStaff getLoginStaff() {
    UcStaff ucStaff = CommonUser.currentUser();
    return getStaffById(ucStaff.getStaffId());
  }
  
  public UcStaff getStaffById(Long staffId) {
    UcStaffExample example = new UcStaffExample();
    example.createCriteria().andStaffIdEqualTo(staffId).andIsDelEqualTo(IsDelEnum.AVAILABLE);
    List<UcStaff> ucStaffs = this.ucStaffMapper.selectByExample(example);
    if (!CollectionUtils.isEmpty(ucStaffs) && ucStaffs.size() == 1)
      return ucStaffs.get(0); 
    return null;
  }
  
  private void removeTokenAndPermissionCache(List<Long> staffIds) {
    this.authenticationService.removeTokenAndPermissionCache(staffIds);
  }
  
  private String getValue(String redisOrgKey) {
    log.info("生成staffCode开始");
    long value = this.redisUtil.incr(redisOrgKey, 1L);
    String staffCode = Tools.formatDateToStringyMd(new Date()) + value;
    if (getStaffByStaffCode(staffCode)) {
      log.info("生成staffCode重复，重新生成");
      getValue(redisOrgKey);
    } 
    log.info("生成staffCode结束");
    return staffCode;
  }
  
  private boolean getStaffByStaffCode(String staffCode) {
    UcStaffExample example = new UcStaffExample();
    example.createCriteria().andStaffCodeEqualTo(staffCode).andIsDelEqualTo(IsDelEnum.AVAILABLE);
    List<UcStaff> ucStaffs = this.ucStaffMapper.selectByExample(example);
    if (CollectionUtils.isEmpty(ucStaffs))
      return false; 
    return true;
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public boolean logicDelStaffByStaffId(Long staffId) {
    UcStaff ucStaff = getStaffById(staffId);
    boolean b = deleteStaff(ucStaff);
    if (b) {
      try {
        JSONObject delMasterRelaResult = this.bssService.delMasterTenantStaffRela(staffId);
        if (!delMasterRelaResult.containsKey("code") || 200 != delMasterRelaResult.getInteger("code").intValue()) {
          log.warn("调用远程接口删除租户与用户关联失败，staffId:{},结果：{}", staffId, delMasterRelaResult);
          throw new BusinessException(FailCode.API_EXCEPTION.getCode(), delMasterRelaResult.getString("msg"));
        } 
        removeTokenAndPermissionCache(Collections.singletonList(staffId));
      } catch (Exception e) {
        log.info("删除员工清空token缓存失败staffId:{},异常:{}", staffId, e);
      } 
      return true;
    } 
    return false;
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public UcStaff logicDelStaffByLoginName(String loginName) {
    UcStaff ucStaff = getStaffByLoginName(loginName);
    if (ucStaff == null)
      throw new BusinessException(FailCode.UPDATE_PASSWORD_STAFF_NONE.getCode(), FailCode.UPDATE_PASSWORD_STAFF_NONE.getMessage()); 
    Long staffId = ucStaff.getStaffId();
    UcStaff loginStaff = getLoginStaff();
    UcStaff staff = validParams(loginStaff, staffId, null, false, null, true, false);
    boolean deleteStaff = deleteStaff(staff);
    if (deleteStaff) {
      JSONObject delMasterRelaResult = this.bssService.delMasterTenantStaffRela(staffId);
      if (!delMasterRelaResult.containsKey("code") || 200 != delMasterRelaResult.getInteger("code").intValue()) {
        log.warn("调用远程接口删除租户与用户关联失败，staffId:{},结果：{}", staffId, delMasterRelaResult);
        throw new BusinessException(FailCode.API_EXCEPTION.getCode(), delMasterRelaResult.getString("msg"));
      } 
    } else {
      return null;
    } 
    try {
      removeTokenAndPermissionCache(Collections.singletonList(staffId));
    } catch (Exception e) {
      log.info("删除员工清空token缓存失败staffId:{},异常:{}", staffId, e);
    } 
    return ucStaff;
  }
  
  @Transactional(rollbackFor = {Exception.class})
  boolean deleteStaff(UcStaff staff) {
    if (null != staff) {
      Long staffId = staff.getStaffId();
      UcStaff record = new UcStaff();
      record.setStaffId(staffId);
      record.setIsDel(IsDelEnum.DELETE);
      record.setLastModifyBy((null == CommonUser.currentUser()) ? "" : CommonUser.currentUser().getLoginName());
      record.setLastModifyTime(LocalDateTimeUtil.UDateToLocalDateTime(new Date()));
      int result = this.ucStaffMapper.updateByPrimaryKeySelective(record);
      if (result > 0) {
        boolean delResult = this.ucStaffOrgService.delAllStaffOrgByStaffId(staffId);
        if (!delResult) {
          log.info("删除用户与组织关联失败");
          throw new BusinessException(FailCode.STAFF_ORG_DEL_BIND_FAIL_FROM4A.getCode(), FailCode.STAFF_ORG_DEL_BIND_FAIL_FROM4A
              .getMessage());
        } 
        log.info("删除用户数据成功，staffId:{}", staffId);
        this.orgTreeCache.delOrgTreeCache();
        return true;
      } 
      log.info("删除用户数据失败，staffId:{}", staffId);
    } 
    return false;
  }
  
  public UcStaff getStaffByLoginName(String loginName) {
    UcStaffExample example = new UcStaffExample();
    example.createCriteria().andLoginNameEqualTo(loginName).andIsDelEqualTo(IsDelEnum.AVAILABLE);
    List<UcStaff> ucStaffs = this.ucStaffMapper.selectByExample(example);
    if (CollectionUtils.isEmpty(ucStaffs)) {
      log.info("获取不到用户信息,loginName:{}", loginName);
      return null;
    } 
    if (ucStaffs.size() == 1)
      return ucStaffs.get(0); 
    log.info("获取到多个用户信息,loginName:{}", loginName);
    throw new BusinessException(FailCode.LOGIN_NAME_UNUSUAL.getCode(), FailCode.LOGIN_NAME_UNUSUAL.getMessage());
  }
  
  public UcStaff validParams(UcStaff loginStaff, Long staffId, Short staffStatus, boolean validStaffStatus, Long roleId, boolean del, boolean udp) {
    if (loginStaff == null) {
      if (validStaffStatus) {
        log.info("启用禁用操作登录用户不存在staffId:{},staffStatus:{}", staffId, staffStatus);
      } else {
        log.info("设置权限操作登录用户不存在staffId:{},roleId:{}", staffId, roleId);
      } 
      throw new BusinessException(FailCode.QUERY_STAFF_LOGIN_STAFFID.getCode(), FailCode.QUERY_STAFF_LOGIN_STAFFID.getMessage());
    } 
    if (StaffStatusEnum.FORBID.equals(loginStaff.getStaffStatus())) {
      if (validStaffStatus) {
        log.info("启用禁用操作登录用户已禁用loginStaffId:{},staffId:{}", loginStaff.getStaffId(), staffId);
      } else {
        log.info("设置权限操作登录用户已禁用loginStaffId:{},staffId:{}", loginStaff.getStaffId(), staffId);
      } 
      throw new BusinessException(FailCode.UPDATE_PASSWORD_STAFF_STATUS.getCode(), FailCode.UPDATE_PASSWORD_STAFF_STATUS.getMessage());
    } 
    if (loginStaff.getRoleId() == null || loginStaff.getRoleId().longValue() == 0L) {
      if (validStaffStatus) {
        log.info("启用禁用操作登录用户是小白无权操作loginStaffId:{},staffId:{}", loginStaff.getStaffId(), staffId);
      } else {
        log.info("设置权限操作登录用户是小白无权操作loginStaffId:{},staffId:{}", loginStaff.getStaffId(), staffId);
      } 
      throw new BusinessException(FailCode.QUERY_STAFF_NONE_PERMISSION.getCode(), FailCode.QUERY_STAFF_NONE_PERMISSION.getMessage());
    } 
    String loginUserPlatFormPermission = this.adminRoleService.getUserPlatform(loginStaff.getStaffId());
    boolean condition1 = "all".equals(loginUserPlatFormPermission);
    if (validStaffStatus) {
      if (!StaffStatusEnum.FORBID.isSame(staffStatus) && !StaffStatusEnum.USE.isSame(staffStatus)) {
        log.info("启用禁用入参操作状态不正确loginStaffId:{},staffId:{},staffStatus:{}", loginStaff.getStaffId(), staffId, staffStatus);
        throw new BusinessException(FailCode.QUERY_STAFF_FAIL_STATUS.getCode(), FailCode.QUERY_STAFF_FAIL_STATUS.getMessage());
      } 
    } else if (roleId != null) {
      AdminRole role = this.adminRoleService.getRoleByRoleId(roleId);
      if (role == null) {
        log.info("设置权限操作role信息不存在,loginStaffId:{},staffId:{},roleId:{}", loginStaff.getStaffId(), staffId, roleId);
        throw new BusinessException(FailCode.STAFF_UPDATE_ROLE_FAIL.getCode(), FailCode.STAFF_UPDATE_ROLE_FAIL.getMessage());
      } 
      String platFormScope = role.getPlatformScope();
      log.info("当前登录用户staffid:{}-权限:{}，被操作用户id:{}-被设置的权限:{}", loginStaff.getStaffId(), loginUserPlatFormPermission, staffId, platFormScope);
      boolean optCondition = (StringUtil.isNotEmpty(platFormScope) && platFormScope.equals(loginUserPlatFormPermission));
      if (!condition1 && !optCondition) {
        log.info("设置权限操作被设置的权限与当前登录人平台不一致,loginStaffId:{}-loginUserPlatFormPermission:{},staffId:{}-platFormScope:{}", loginStaff.getStaffId(), loginUserPlatFormPermission, staffId, platFormScope);
        throw new BusinessException(FailCode.STAFF_BIND_ROLE_FAIL.getCode(), FailCode.STAFF_BIND_ROLE_FAIL.getMessage());
      } 
    } 
    if (loginStaff.getStaffId() != null && staffId != null && loginStaff.getStaffId().compareTo(staffId) == 0) {
      if (validStaffStatus) {
        log.info("启用禁用操作不可操作自己,loginStaffId:{},staffId:{},staffType:{}", loginStaff.getStaffId(), staffId, loginStaff.getStaffType());
      } else {
        log.info("设置权限操作不可操作自己,loginStaffId:{},staffId:{},staffType:{}", loginStaff.getStaffId(), staffId, loginStaff.getStaffType());
      } 
      if (!udp)
        throw new BusinessException(FailCode.QUERY_STAFF_UNABLE_OWN.getCode(), FailCode.QUERY_STAFF_UNABLE_OWN.getMessage()); 
    } 
    UcStaff staff = getStaffByIdIgnoreStatus(staffId);
    if (staff == null) {
      if (validStaffStatus) {
        log.info("启用禁用操作用户不存在，loginStaffId:{},staffId:{}", loginStaff.getStaffId(), staffId);
      } else {
        log.info("设置权限操作用户不存在loginStaffId:{},staffId:{}", loginStaff.getStaffId(), staffId);
      } 
      throw new BusinessException(FailCode.UPDATE_PASSWORD_STAFF_NONE.getCode(), FailCode.UPDATE_PASSWORD_STAFF_NONE.getMessage());
    } 
    if (Long.valueOf("0").equals(staff.getRoleId()) || staff.getRoleId() == null)
      if (validStaffStatus) {
        log.info("启用禁用操作被操作人是小白,loginStaffId:{},staffId:{},roleId:{}", loginStaff.getStaffId(), staffId, (staff.getRoleId() == null) ? "" : staff.getRoleId());
      } else {
        log.info("设置权限操作被操作人是小白,loginStaffId:{},staffId:{},roleId:{}", loginStaff.getStaffId(), staffId, (staff.getRoleId() == null) ? "" : staff.getRoleId());
      }  
    return staff;
  }
  
  private UcStaff getStaffByIdIgnoreStatus(Long staffId) {
    UcStaffExample example = new UcStaffExample();
    example.createCriteria().andStaffIdEqualTo(staffId).andIsDelEqualTo(IsDelEnum.AVAILABLE);
    List<UcStaff> ucStaffs = this.ucStaffMapper.selectByExample(example);
    if (!CollectionUtils.isEmpty(ucStaffs) && ucStaffs.size() == 1)
      return ucStaffs.get(0); 
    return null;
  }
  
  public boolean sendEmail(EmailDto emailDto) {
    Long staffId = emailDto.getStaffId();
    UcStaff staff = getStaffById(staffId);
    if (null == staff) {
      log.warn("发送邮件，当前用户不存在，staffId：{}", staffId);
      return false;
    } 
    if (StringUtils.isEmpty(staff.getStaffEmail())) {
      log.warn("发送邮件，用户信息中无邮箱信息，staffId：{}", staffId);
      return false;
    } 
    String email = staff.getStaffEmail();
    String subject = emailDto.getSubject();
    String content = emailDto.getContent();
    if (content.contains("%s"))
      content = String.format(content, new Object[] { staff.getStaffName() }); 
    ByteArrayOutputStream os = emailDto.getOs();
    String attachmentFilename = emailDto.getAttachmentFilename();
    Map<String, byte[]> attachmentFileMap = emailDto.getAttachmentFileMap();
    try {
      if (attachmentFileMap == null || attachmentFileMap.isEmpty()) {
        this.mailService.sendMail(email, subject, content, os, attachmentFilename);
      } else {
        this.mailService.sendMail(email, subject, content, attachmentFileMap);
      } 
    } catch (Exception e) {
      log.error("发送邮件异常", e);
      return false;
    } 
    return true;
  }
  
  public UcStaff checkStaffByLoginName(String loginName, String password) {
    UcStaffExample ex = new UcStaffExample();
    ex.createCriteria()
      .andLoginNameEqualTo(loginName)
      .andIsDelEqualTo(IsDelEnum.AVAILABLE)
      .andStaffStatusEqualTo(StaffStatusEnum.USE);
    List<UcStaff> ucStaffList = this.ucStaffMapper.selectByExample(ex);
    if (CollectionUtils.isEmpty(ucStaffList)) {
      log.error("用户{}校验失败，{}", loginName, FailCode.LOGIN_NAME_NO_EX.getMessage());
      return null;
    } 
    if (ucStaffList.size() > 1) {
      log.warn("用户{}校验失败，{}", loginName, FailCode.LOGIN_NAME_UNUSUAL.getMessage());
      return null;
    } 
    UcStaff ucStaff = ucStaffList.get(0);
    String encryptPassword = PasswordUtils.getEncryptPassword(password, ucStaff.getSalt(), this.encryptType);
    if (!encryptPassword.equals(ucStaff.getPassword())) {
      log.warn("用户{}校验失败，密码错误", loginName);
      return null;
    } 
    return ucStaff;
  }
  
  private void addStaffOrgPermission(StaffOrgPermissionDto staffOrgPermissionDto) {
    Long currentUser = CommonUser.getAuthStaffDto().getStaffId();
    Long staffId = staffOrgPermissionDto.getStaffId();
    List<OrgTenantDto> orgTenantList = staffOrgPermissionDto.getOrgTenantList();
    List<AdminStaffOrg> adminStaffOrgList = Lists.newArrayList();
    for (OrgTenantDto orgTenantDto : orgTenantList) {
      AdminStaffOrg adminStaffOrg = new AdminStaffOrg();
      adminStaffOrg.setStaffId(staffId);
      adminStaffOrg.setOrgId(orgTenantDto.getOrgId());
      adminStaffOrg.setMasterTenantId(orgTenantDto.getMasterTenantId());
      adminStaffOrg.setTenantId(orgTenantDto.getTenantId());
      adminStaffOrg.setCreateBy(currentUser.toString());
      adminStaffOrgList.add(adminStaffOrg);
    } 
    this.adminStaffOrgMapper.insertBatch(adminStaffOrgList);
  }
  
  public void deleteStaffOrgPermission(AdminStaffOrg adminStaffOrg) {
    List<AdminStaffOrg> adminStaffOrgList = this.adminStaffOrgMapper.selectByCondition(adminStaffOrg);
    try {
      removeTokenAndPermissionCache((List<Long>)adminStaffOrgList.stream().map(AdminStaffOrg::getStaffId).collect(Collectors.toList()));
    } catch (Exception e) {
      log.info("删除员工清空token缓存失败staffId:{},异常:{}", adminStaffOrg, e);
    } 
    this.adminStaffOrgMapper.delete(adminStaffOrg);
  }
  
  @Transactional(rollbackFor = {Exception.class})
  public void updStaffOrgPermission(StaffOrgPermissionDto staffOrgPermissionDto) {
    AdminStaffOrg adminStaffOrg = new AdminStaffOrg();
    adminStaffOrg.setStaffId(staffOrgPermissionDto.getStaffId());
    deleteStaffOrgPermission(adminStaffOrg);
    addStaffOrgPermission(staffOrgPermissionDto);
  }
  
  public List<OrgTenantDto> getStaffOrgPermissionList(Long staffId) {
    if (staffId == null)
      return null; 
    AdminStaffOrg record = new AdminStaffOrg();
    record.setStaffId(staffId);
    List<AdminStaffOrg> adminStaffOrgList = this.adminStaffOrgMapper.selectByCondition(record);
    return (List<OrgTenantDto>)adminStaffOrgList.stream().map(adminStaffOrg -> {
          OrgTenantDto dto = new OrgTenantDto();
          dto.setTenantId(adminStaffOrg.getTenantId());
          dto.setMasterTenantId(adminStaffOrg.getMasterTenantId());
          dto.setOrgId(adminStaffOrg.getOrgId());
          return dto;
        }).collect(Collectors.toList());
  }
  
  public UcStaff getStaffByWechat(String wechatSign) {
    return this.ucStaffMapper.getStaffByWechat(wechatSign);
  }
  
  public Long getStaffIdByBimUid(String bimUid) {
    return this.ucStaffMapper.getStaffIdByBimUid(bimUid);
  }
  
  public UcStaff convertToStaff(StaffDto staffDto) {
    UcStaff ucStaff = new UcStaff();
    ucStaff.setStaffId(staffDto.getStaffId());
    ucStaff.setUserId(staffDto.getUserId());
    ucStaff.setStaffCode(staffDto.getLoginName());
    ucStaff.setStaffType(staffDto.getStaffType());
    ucStaff.setStaffName(staffDto.getStaffName());
    ucStaff.setLoginName(staffDto.getLoginName());
    ucStaff.setIdcardNo(staffDto.getKey());
    if (staffDto.getStaffFrom() != null && staffDto.getStaffFrom().shortValue() == SourceFromEnum.FROM4A.value().shortValue()) {
      ucStaff.setStaffFrom(SourceFromEnum.FROM4A);
    } else {
      ucStaff.setStaffFrom(SourceFromEnum.CREATE);
    } 
    if (staffDto.getIsDel() != null && staffDto.getIsDel().shortValue() == IsDelEnum.DELETE.value().shortValue()) {
      ucStaff.setIsDel(IsDelEnum.DELETE);
    } else {
      ucStaff.setIsDel(IsDelEnum.AVAILABLE);
    } 
    ucStaff.setDefaultPolicy("true".equals(staffDto.getIsDefaultPolicy()));
    ucStaff.setStaffEmail(staffDto.getStaffEmail());
    ucStaff.setStaffPhone(staffDto.getStaffPhone());
    ucStaff.setPhoneUserCenter(staffDto.getStaffPhone());
    if (staffDto.getStaffStatus() != null && staffDto.getStaffStatus().shortValue() == StaffStatusEnum.FORBID.value().shortValue()) {
      ucStaff.setStaffStatus(StaffStatusEnum.FORBID);
    } else {
      ucStaff.setStaffStatus(StaffStatusEnum.USE);
    } 
    if (!StringUtils.isEmpty(staffDto.getPassword())) {
      String newSalt = SaltUtils.getSalt();
      ucStaff.setSalt(newSalt);
      String encryptPassword = PasswordUtils.getEncryptPassword(staffDto.getPassword(), newSalt, this.encryptType);
      ucStaff.setPassword(encryptPassword);
    } 
    if (staffDto.getStaffStatus() != null && staffDto.getStaffStatus().shortValue() == StaffStatusEnum.FORBID.value().shortValue()) {
      ucStaff.setStaffStatus(StaffStatusEnum.FORBID);
    } else {
      ucStaff.setStaffStatus(StaffStatusEnum.USE);
    } 
    ucStaff.setExt1(staffDto.getExt1());
    ucStaff.setRoleId(staffDto.getRoleId());
    ucStaff.setVerifyCode(staffDto.getVerifyCode());
    ucStaff.setExt2(staffDto.getMultiFactorAuth());
    return ucStaff;
  }
  
  public List<UcStaffOrg> convertToStaffOrg(StaffDto staffDto) {
    List<UcStaffOrg> list = new ArrayList<>();
    UcStaffOrg staffOrg = new UcStaffOrg();
    staffOrg.setOrgCode(staffDto.getOrgCode());
    if (StringUtils.isEmpty(staffDto.getOrgName())) {
      staffOrg.setOrgName(staffDto.getOrgCode());
    } else {
      staffOrg.setOrgName(staffDto.getOrgName());
    } 
    list.add(staffOrg);
    return list;
  }
  
  public StaffDataPermission getStaffDataPermission(AuthUcStaffDto staffDto) {
    return getStaffDataPermission(staffDto, Integer.valueOf(0));
  }
  
  public StaffDataPermission getStaffDataPermission(AuthUcStaffDto staffDto, Integer type) {
    StaffDataPermission staffDataPermission = new StaffDataPermission();
    staffDataPermission.setStaffId(staffDto.getStaffId());
    String roleType = staffDto.getRoleType();
    AdminRoleEnum targetRoleEnum = AdminRoleEnum.getAdminRole(staffDto.getRoleTag());
    Integer roleTypeInt = Integer.valueOf(roleType);
    staffDataPermission.setStaffPermissionType(roleTypeInt);
    if (type.intValue() != 1 && AdminRoleEnum.ADMIN.getPermissionType().equals(roleTypeInt))
      return staffDataPermission; 
    JSONObject result = this.bssService.getAllMasterTenantList();
    if (!result.containsKey("code") || !Constant.CODE_SUCCESS.equals(result.getInteger("code"))) {
      log.error("获取全部平台租户树失败，结果：{}", result);
      throw new BusinessException(FailCode.API_EXCEPTION.getCode(), result.getString("info"));
    } 
    List<MasterTenantVo> masterTenantAllList = result.getJSONArray("data").toJavaList(MasterTenantVo.class);
    List<StaffOrganization> staffOrganizationList = new ArrayList<>(8);
    if (type.intValue() == 1 && AdminRoleEnum.ADMIN.getPermissionType().equals(roleTypeInt)) {
      setAllStaffMaster(masterTenantAllList, staffOrganizationList);
      staffDataPermission.setPermissionList(staffOrganizationList);
      return staffDataPermission;
    } 
    if (targetRoleEnum == AdminRoleEnum.ORG_ADMIN) {
      List<MasterTenantVo> masterTenantVos = warpMasterTenantTree(masterTenantAllList, 
          Long.valueOf(staffDto.getMasterTenantId()));
      setAllStaffMaster(masterTenantVos, staffOrganizationList);
      staffDataPermission.setPermissionList(staffOrganizationList);
      return staffDataPermission;
    } 
    List<OrgTenantDto> staffOrgPermissionList = getStaffOrgPermissionList(staffDto.getStaffId());
    Map<Long, List<OrgTenantDto>> masterTenantMap = (Map<Long, List<OrgTenantDto>>)staffOrgPermissionList.stream().collect(Collectors.groupingBy(OrgTenantDto::getMasterTenantId));
    List<MasterTenantVo> permissionMasterTenantList = (List<MasterTenantVo>)masterTenantAllList.stream().filter(a -> masterTenantMap.containsKey(a.getMasterTenantId())).collect(Collectors.toList());
    for (MasterTenantVo masterTenantVo : permissionMasterTenantList) {
      List<Long> tenantIdList = (List<Long>)((List)masterTenantMap.getOrDefault(masterTenantVo.getMasterTenantId(), Lists.newArrayList())).stream().map(OrgTenantDto::getTenantId).collect(Collectors.toList());
      StaffOrganization staffOrganization = getStaffOrganizationByMaster(masterTenantVo, tenantIdList);
      staffOrganizationList.add(staffOrganization);
    } 
    staffDataPermission.setPermissionList(staffOrganizationList);
    return staffDataPermission;
  }
  
  private static List<MasterTenantVo> warpMasterTenantTree(List<MasterTenantVo> list, Long firstMasterTenantId) {
    Map<Long, List<MasterTenantVo>> collect = (Map<Long, List<MasterTenantVo>>)list.stream().filter(a -> (0L != a.getParentId().longValue())).collect(Collectors.groupingBy(MasterTenantVo::getParentId));
    for (MasterTenantVo masterTenantVo : list)
      masterTenantVo.setChildren(collect.get(masterTenantVo.getMasterTenantId())); 
    return (List<MasterTenantVo>)list.stream().filter(a -> firstMasterTenantId.equals(a.getMasterTenantId())).collect(Collectors.toList());
  }
  
  private void setAllStaffMaster(List<MasterTenantVo> masterTenantVoList, List<StaffOrganization> staffOrganizationList) {
    for (MasterTenantVo masterTenantVo : masterTenantVoList) {
      StaffOrganization staffOrganization = getStaffOrganizationByMaster(masterTenantVo);
      staffOrganizationList.add(staffOrganization);
      if (!CollectionUtils.isEmpty(masterTenantVo.getChildren()))
        setAllStaffMaster(masterTenantVo.getChildren(), staffOrganizationList); 
    } 
  }
  
  private StaffOrganization getStaffOrganizationByMaster(MasterTenantVo vo) {
    StaffOrganization staffOrganization = new StaffOrganization();
    staffOrganization.setOrgCode(vo.getOrgCode());
    staffOrganization.setOrgName(vo.getOrgName());
    staffOrganization.setMasterTenantId(vo.getMasterTenantId() + "");
    staffOrganization.setMasterTenantName(vo.getMasterTenantName());
    staffOrganization.setMasterTenantStatus(vo.getStatus());
    staffOrganization.setTenantList(vo.getTenantList());
    return staffOrganization;
  }
  
  private StaffOrganization getStaffOrganizationByMaster(MasterTenantVo vo, List<Long> tenantIdList) {
    StaffOrganization staffOrganization = getStaffOrganizationByMaster(vo);
    if (!CollectionUtils.isEmpty(vo.getTenantList()) && !CollectionUtils.isEmpty(tenantIdList)) {
      List<PermissionTenant> permissionTenantList = (List<PermissionTenant>)vo.getTenantList().stream().filter(a -> tenantIdList.contains(a.getTenantId())).collect(Collectors.toList());
      staffOrganization.setTenantList(permissionTenantList);
    } 
    return staffOrganization;
  }
  
  public AdminRoleEnum checkStaffOpPermission(Long adminRoleId) {
    StaffDataPermission staffDataPermission = this.permission.checkDataPermission();
    AdminRole role = this.adminRoleService.getRoleByRoleId(adminRoleId);
    AdminRoleEnum targetRoleEnum = AdminRoleEnum.getAdminRole(role.getTag());
    if (AdminRoleEnum.USER.getPermissionType().equals(staffDataPermission.getStaffPermissionType()))
      throw new BusinessException(FailCode.DATA_PERMISSION_AUTH_FAIL); 
    if (!AdminRoleEnum.ADMIN.getPermissionType().equals(staffDataPermission.getStaffPermissionType()) && targetRoleEnum != null)
      throw new BusinessException(FailCode.DATA_PERMISSION_AUTH_FAIL); 
    return targetRoleEnum;
  }
  
  public List<OrgTenantDto> getStaffOrgPermissionListForSso(Long staffId) {
    if (staffId == null)
      throw new BusinessException(FailCode.ROLE_ID_EMPTY); 
    Map<Object, Object> param = new HashMap<>();
    param.put("staffId", staffId);
    List<Map> adminStaffOrgList = this.adminStaffOrgMapper.selectByConditionForSso(param);
    return (List<OrgTenantDto>)adminStaffOrgList.stream()
      .filter(adminStaffOrg -> (MapUtils.getLong(adminStaffOrg, "tenantId").longValue() > 0L))
      .map(adminStaffOrg -> {
          OrgTenantDto dto = new OrgTenantDto();
          dto.setTenantId(MapUtils.getLong(adminStaffOrg, "tenantId"));
          dto.setMasterTenantId(MapUtils.getLong(adminStaffOrg, "masterTenantId"));
          dto.setSsoAccountId(MapUtils.getString(adminStaffOrg, "ssoAccountId"));
          return dto;
        }).collect(Collectors.toList());
  }
  
  public int updateStaffInfo(UcStaff ucStaff) {
    log.info("OA同步修改用户信息：{}", JSONObject.toJSONString(ucStaff));
    ucStaff.setLastModifyBy("OA");
    ucStaff.setLastModifyTime(LocalDateTimeUtil.UDateToLocalDateTime(new Date()));
    return this.ucStaffMapper.updateByPrimaryKeySelective(ucStaff);
  }
  
  public String decryptAesPwd(String aesPwd) {
    if (aesPwd == null)
      return null; 
    try {
      return AESUtil.decryptAES(aesPwd.getBytes(), this.aesKey, this.aesIv);
    } catch (Exception e) {
      log.error("密码解密异常 param【{}】", aesPwd, e);
      throw new BusinessException(FailCode.PASSWORD_DECRYPT_FAILURE);
    } 
  }
}
